import numpy as np
from scipy.interpolate import griddata

def create_grid(lonmin, lonmax, latmin, latmax, **kwargs):
    dx, dy = kwargs.get("dx", None), kwargs.get("dy", None)
    nx, ny = kwargs.get("nx", None), kwargs.get("ny", None)
    if nx==None or ny==None:
        if dx==None or dy==None:
            raise KeyError("Please kwargs dx and dy to define the interval between grid points, or nx and ny to define the number of grid points (include endpoints) between the bbox limits")
        else:
            nx, ny = (lonmax-lonmin)/dx, (latmax-latmin)/dy
    lat_grid = np.linspace(latmin, latmax, ny, endpoint=True)
    lon_grid = np.linspace(lonmin, lonmax, nx, endpoint=True)
    return lon_grid, lat_grid

class GenInterpolator(object):
    def __init__(self, data, *dimensions, **kwargs):
        for dim in dimensions:
            assert len(dim.shape) == 1
        method = kwargs.get('method', 'nearest')
        dimensions = [dim.flatten() for dim in dimensions]
        points = [dim.flatten() for dim in np.meshgrid(*dimensions)]
        points = np.asarray(points).T
        self.points = points
        self.data = data.flatten()
        self.method = method
        self.numdim = len(dimensions)

    def interpgrid(self, *dimensions):
        if len(dimensions) != self.numdim:
            raise ValueError("Please interpolate data to the same number of dimensions as source data.")
        else:
            for dim in dimensions:
                assert len(dim.shape) == 1
            flatdims = [dim.flatten() for dim in dimensions]
            newpoints = [dim.flatten() for dim in np.meshgrid(*flatdims)]
            newpoints = np.asarray(newpoints).T
            f = griddata(self.points, self.data, newpoints, method=self.method)
            unique = []
            for dim in dimensions:
                unique.append(dim.shape[0])
            return np.squeeze( f.reshape( *unique ) )

class CfGeoInterpolator(object):
    def __init__(self, data, lon, lat, t=None, z=None, **kwargs):
        method = kwargs.get('method', 'nearest')
        coords = {'lat':lat, 'lon':lon, 'z':z, 't':t}
        dimensions, ndshape = self._flatten_coords(**coords)
        self.points = np.asarray(dimensions).T
        self.data = data.flatten()
        self.method = method
        self.numdim = self.points.shape[1]
        assert self.data.shape[0] == self.points.shape[0]

    def interpgrid(self, lon, lat, t=None, z=None, **kwargs):
        coords = {'lat':lat, 'lon':lon, 'z':z, 't':t}
        dimensions, ndshape = self._flatten_coords(**coords)
        dimensions = np.asarray(dimensions).T
        #print self.points.shape, self.data.shape, dimensions.shape
        #print np.histogram(self.data)
        f = griddata(self.points, self.data, dimensions, method=self.method)
        return np.squeeze( f.reshape( *ndshape ) )

    def _flatten_coords(self, **coords):
        lat = coords.get('lat', None)
        lon = coords.get('lon', None)
        z = coords.get('z', None)
        t = coords.get('t', None)
        ndshape = []

        # Configure latitude and longitude to produce vectors of cell by
        # cell coordinates
        if len(lat.shape) == 2:
            assert np.all(lat.shape==lon.shape)
            ndshape.append(lat.shape[1])
            ndshape.append(lat.shape[0])
            latshape = lat.shape
            lat = lat.flatten()
            lon = lon.flatten()
        else:
            assert len(lat.shape) == 1
            assert len(lon.shape) == 1
            ndshape.append(lon.shape[0])
            ndshape.append(lat.shape[0])
            lon, lat = np.meshgrid(lon, lat, indexing='ij')
            latshape = lat.shape
            lon = lon.flatten()
            lat = lat.flatten()

        # Configure the z coords to provide cell by cell z value
        if z == None:
            if t == None:
                dimensions = [lon, lat]
            else:
                ndshape.append(t.shape[0])
                lat = np.meshgrid(t, lat, indexing='ij')[-1]
                t, lon = np.meshgrid(t, lon, indexing='ij')
                dimensions = [lon.flatten(), lat.flatten(), t.flatten()]
        elif len(z.shape) == 4:
            assert t != None
            ndshape.append(z.shape[1])
            ndshape.append(t.shape[0])
            lat = np.meshgrid(t, range(z.shape[1]), lat, indexing='ij')[-1]
            t, dummy, lon = np.meshgrid(t, range(z.shape[1]), lon, indexing='ij')
            z = z.flatten()
            dimensions = [lon.flatten(), lat.flatten(), z, t.flatten()]
        elif len(z.shape) ==  3:
            assert np.all(z.shape[1:] == latshape)
            if t == None:
                ndshape.append(z.shape[0])
                lat = np.meshgrid(range(z.shape[0]), lat, indexing='ij')[-1]
                lon = np.meshgrid(range(z.shape[0]), lon, indexing='ij')[-1]
                z = z.flatten()
                dimensions = [lon.flatten(), lat.flatten(), z]
            else:
                ndshape.append(z.shape[0])
                ndshape.append(t.shape[0])
                lat = np.meshgrid(t, range(z.shape[0]), lat, indexing='ij')[-1]
                lon = np.meshgrid(t, range(z.shape[0]), lon, indexing='ij')[-1]
                t, z = np.meshgrid(t, z.flatten(), indexing='ij')
                dimensions = [lon.flatten(), lat.flatten(), z.flatten(), t.flatten()]
        elif len(z.shape) == 1:
            if t == None:
                ndshape.append(z.shape[0])
                lat = np.meshgrid(z, lat, indexing='ij')[-1]
                z, lon = np.meshgrid(z, lon, indexing='ij')
                dimensions = [lon.flatten(), lat.flatten(), z.flatten()]
            else:
                ndshape.append(z.shape[0])
                ndshape.append(t.shape[0])
                lat = np.meshgrid(t, z, lat, indexing='ij')[-1]
                t, z, lon = np.meshgrid(t, z, lon, indexing='ij')
                dimensions = [lon.flatten(), lat.flatten(), z.flatten(), t.flatten()]

        return dimensions, ndshape[::-1]



import numpy as np
import netCDF4 as ncd

FILL_VALUE = None

def new(filename):
    '''
    Return the netcdf4-python rootgroup for a new netcdf file
    for writing.
    '''
    return ncd.Dataset(filename, 'w', clobber=False)

def add_coordinates(nc, dict_of_dims):
    '''
    Create dimensions in netcdf file nc.

    >> add_coordinates(nc, {"time", (5040,)})
    '''
    # Loop through keys, and add each as a dimension, with the
    # cooresponding dict value tuple as the representative
    # shape of the dimension.
    for dimname in dict_of_dims.iterkeys():
        t = nc.createDimension(dimname, size=dict_of_dims[dimname])
    nc.sync()
    
def add_variable(nc, varname, data, dims, compress=False, fill=FILL_VALUE):
    '''
    Thin wrapper for easily adding data to netcdf variable with just
    the variable name the current array of values, and a tuple with
    the cooresponding dimension names
    '''
    v = nc.createVariable(varname, data.dtype, dimensions=dims, zlib=compress, fill_value=fill) 
    v[:] = data
    nc.sync()

def add_scalar(nc, varname, data, compress=False, fill=FILL_VALUE):
    '''
    Functionality to simply add a scalar variable to a netcdf file,
    expects a numpy array of length 1 for the data argument.
    '''
    v = nc.createVariable(varname, data.dtype, zlib=compress, fill_value=fill)
    v[:] = data
    nc.sync()

def add_attribute(nc, key, value, var=None):
    '''
    Take in a single attname:value pair and write into the global
    or variable namespace
    '''
    if var==None:
        nc.setncattr(key, value)
    else:
        if not key in set([ "_FillValue", "_ChunkSize" ]):
            nc.variables[var].setncattr(key, value)
    nc.sync()

def add_attributes(nc, attrs, var=None):
    '''
    Take in a dict of attname:value pairs and write the whole dict
    into the global or variable namespace
    '''
    if var==None:
        nc.setncatts(attrs)
    else:
        nc.variables[var].setncatts(attrs)
    nc.sync()



import os.path
import threading
import numpy as np
import netCDF4
from paegan.utils.asainterpolate import CfGeoInterpolator
import paegan.cdm.writer as pw
from collections import OrderedDict
import datetime

#       ---------------------------------
#       rho | u | rho | u | rho | u | rho
#       ---------------------------------
#        v  |   | {v} |   | {v} |   |  v
#       ---------------------------------
#       rho |{u}|(rho)|{u}|(rho)|{u}| rho
#       ---------------------------------
#        v  |   | {v} |   | {v} |   |  v
#       ---------------------------------
#       rho | u | rho | u | rho | u | rho
#       ---------------------------------
#
#       Assumes:
#       size(rho)=ny,nx;   size(u)=ny,nx-1;   size(v)=ny-1,nx
#       coordinate dimension: t,z,y,x
#
#       When Averaging U and V into the RHO grid:
#       * We lose the first and last column of rho and V
#       * We lose the first and last row of rho and U

def shrink(a, b):
    """
    Ripped from Octant: https://code.google.com/p/octant

    Return array shrunk to fit a specified shape by triming or averaging.

    a = shrink(array, shape)

    array is an numpy ndarray, and shape is a tuple (e.g., from
    array.shape). a is the input array shrunk such that its maximum
    dimensions are given by shape. If shape has more dimensions than
    array, the last dimensions of shape are fit.

    as, bs = shrink(a, b)

    If the second argument is also an array, both a and b are shrunk to
    the dimensions of each other. The input arrays must have the same
    number of dimensions, and the resulting arrays will have the same
    shape.

    Example
    -------

    >>> shrink(rand(10, 10), (5, 9, 18)).shape
    (9, 10)
    >>> map(shape, shrink(rand(10, 10, 10), rand(5, 9, 18)))
    [(5, 9, 10), (5, 9, 10)]

    """

    if isinstance(b, np.ndarray):
        if not len(a.shape) == len(b.shape):
            raise Exception, \
                  'input arrays must have the same number of dimensions'
        a = shrink(a,b.shape)
        b = shrink(b,a.shape)
        return (a, b)

    if isinstance(b, int):
        b = (b,)

    if len(a.shape) == 1:                # 1D array is a special case
        dim = b[-1]
        while a.shape[0] > dim:          # only shrink a
            if (dim - a.shape[0]) >= 2:  # trim off edges evenly
                a = a[1:-1]
            else:                        # or average adjacent cells
                a = 0.5*(a[1:] + a[:-1])
    else:
        for dim_idx in range(-(len(a.shape)),0):
            dim = b[dim_idx]
            a = a.swapaxes(0,dim_idx)        # put working dim first
            while a.shape[0] > dim:          # only shrink a
                if (a.shape[0] - dim) >= 2:  # trim off edges evenly
                    a = a[1:-1,:]
                if (a.shape[0] - dim) == 1:  # or average adjacent cells
                    a = 0.5*(a[1:,:] + a[:-1,:])
            a = a.swapaxes(0,dim_idx)        # swap working dim back

    return a

def _uv_to_rho(u_data, v_data, angle, rho_x, rho_y):
    # Create empty rho matrix to store complex U + Vj
    U = np.empty([rho_y,rho_x], dtype=complex )
    # And fill it with with nan values
    U[:] = np.nan

    vfunc = np.vectorize(complex)

    # THREE IDENTICAL METHODS
    # Fill in RHO cells per diagram above.  Skip first row and first
    # column of RHOs and leave them as numpy.nan values.  Also skip the
    # last row and column.

    #1.)
    # Fill rho points with averages (if we can do the calculation)
    # Thread]
    #t1 = AverageAdjacents(u_data)
    #t2 = AverageAdjacents(v_data, True)
    #t1.start(); t2.start()
    #t1.join();  t2.join()
    #u_avg = t1.data
    #v_avg = t2.data
    #complexed = vfunc(u_avg[1:-1,:],v_avg[:,1:-1])

    # 2.)
    # Don't thread
    u_avg = average_adjacents(u_data)
    v_avg = average_adjacents(v_data,True)
    complexed = vfunc(u_avg[1:-1,:],v_avg[:,1:-1])

    # 3.)
    # Use Octant method
    #data_U = (rho_y-2,rho_x-2)
    #u_avg = shrink(u_data, data_U)
    #v_avg = shrink(v_data, data_U)
    #complexed = vfunc(u_avg,v_avg)

    # Only pull the U and V values that can contribute to the averaging (see diagram).
    # This means we lost the first and last row and the first and last column of both
    # U and V.
    U[1:rho_y-1, 1:rho_x-1] = complexed

    # We need the rotated point, so rotate by the "angle"
    return rotate_complex_by_angle(U,angle)

def uv_to_rho(file):
    nc = netCDF4.Dataset(file)

    lat_rho=nc.variables['lat_rho']
    lon_rho=nc.variables['lon_rho']

    # lat_rho and lon_rho should have identical shapes (eta,xi)/(y/x)
    if lat_rho.shape != lon_rho.shape:
        print "Shape of lat_rho and lon_rho must be equal"
        return None

    # Store shape for use below
    [rho_y,rho_x] = lat_rho.shape

    # U
    u_data = nc.variables['u'][0,0,:,:]

    # V
    v_data = nc.variables['v'][0,0,:,:]

    # Get the angles
    angle = nc.variables['angle'][:,:]

    # Close the dataset
    nc.close()

    return _uv_to_rho(u_data, v_data, angle, rho_x, rho_y)

def rotate_complex_by_angle(points,angles):
    """
        points: ND array of complex numbers
        angles: ND array of angles, same size as points

        Multiply the points we have (stored as complex) by another complex number
        whose angle with the positive X axis is the angle we need to rotate by.

        np.exp(1J*angle)  creates an array the same size of U filled with complex numbers
                          that represent the angled points from X axis.
    """
    return points * np.exp(1j*angles)

def average_adjacents(a, by_column=False):
    """
        Sums adjacent values in a column.  Optional by_column parameter
        will sum adjacement column values.

        For a row that looks like this:
        [ 2, 4, 6, 8, 10, 12 ]

        The following addition is computed:

        [ 2, 4, 6,  8, 10 ]
                +
        [ 4, 6, 8, 10, 12 ]

        Resulting in

        [ 3, 5, 7, 9, 11 ]

        Use:

            # 1D Array
            >>> a = np.arange(0,13,2)
            >>> a
            array([ 0,  2,  4,  6,  8, 10, 12])

            >>> a_avg = average_adjacents(a)
            >>> a_avg
            array([  1.,   3.,   5.,   7.,   9.,  11.])


            # 2D Array
            >>> a = np.arange(0,30,2).reshape(3,5)
            >>> a
            array([[ 0,  2,  4,  6,  8],
                   [10, 12, 14, 16, 18],
                   [20, 22, 24, 26, 28]])
            >>> a_avg = average_adjacents(a)
            >>> a_avg
            array([[  1,  3,  5,  7],
                   [ 11, 13, 15, 17],
                   [ 21, 23, 25, 27]])


            # 2D Array, by column
            >>> a = np.arange(0,30,2).reshape(3,5)
            >>> a
            array([[ 0,  2,  4,  6,  8],
                   [10, 12, 14, 16, 18],
                   [20, 22, 24, 26, 28]])
            >>> a_avg = average_adjacents(a, True)
            >>> a_avg
            array([[  5.,   7.,   9.,  11.,  13.],
                   [ 15.,  17.,  19.,  21.,  23.]])

    """
    if by_column:
        # Easier to transpose, sum by row, and transpose back.
        #sumd = 0.5 * (a[1:m,:] + a[0:m-1,:])
        return average_adjacents(a.T).T

    try:
        [m,n] = a.shape
    except ValueError:
        [m,n] = [1,a.size]

    if m == 1:
        sumd = 0.5 * (a[0:n-1] + a[1:n]) # Single row
    else:
        sumd = 0.5 * (a[:,0:n-1] + a[:,1:n]) # By row

    return sumd

def regrid_roms(newfile, filename, lon_new, lat_new, t=None, z=None):
    '''Function to regrid the entire roms datasets (all values on non-rho coords)
       onto an arbitrary grid to support regridding on to regular grids as well.
    '''
    with pw.new(newfile) as new:
        with netCDF4.Dataset(filename) as nc:
            for key in nc.variables:
                try:
                    if "since" in nc.variables[key].units:
                        time = nc.variables[key][:]
                except AttributeError:
                    pass
            # Identify the rho coordinates, and get them
            lon_rho = nc.variables["lon_rho"][:]
            lat_rho = nc.variables["lat_rho"][:]
            rho_y, rho_x = lat_rho.shape
            depth_rho = nc.variables["s_rho"][:]
            #depth_w   = nc.variables["s_w"][:]
            if t == None:
                t = time
            # Put dimensions into the new netcdf file, should only be for
            # time, rhos, psis (not sure what to do about w yet)
            if len(depth_rho.shape) == 4:
                s_rho = depth_rho.shape[1]
            else:
                s_rho = depth_rho.shape[0]
            if len(lon_new.shape) == 2 and len(lon_new.shape) == 2:
                eta_new = lat_new.shape[0]
                xi_new = lon_new.shape[1]
            elif len(lon_new.shape) == 1 and len(lon_new.shape) == 1:
                eta_new = lat_new.shape[0]
                xi_new = lon_new.shape[0]
            else:
                raise ValueError("New lat and lon have invalid shapes or don't match in shape.")
            if z == None:
                z = depth_rho
            s_new = z.shape[0]
            time_new = t.shape[0]

            #[pw.add_attribute(new, at, new.getncattr(at)) for at in new.ncattrs()]
            pw.add_coordinates(new, OrderedDict([("time_new",time_new),("s_new",s_new),("eta_new",eta_new),("xi_new",xi_new)]))
            #print "Coordinates " + str([("time_new",time_new),("s_new",s_new),("eta_new",eta_new),("xi_new",xi_new)])

            pw.add_variable(new, "ocean_time", t, ("time_new",))
            pw.add_variable(new, "s_new", z, ("s_new",))
            if len(lon_new.shape) == 2 and len(lat_new.shape) == 2:
                pw.add_variable(new, "lat_new", lat_new, ("eta_new", "xi_new",))
                pw.add_variable(new, "lon_new", lon_new, ("eta_new", "xi_new",))
            elif len(lon_new.shape) == 1 and len(lat_new.shape) == 1:
                pw.add_variable(new, "lat_new", lat_new, ("eta_new",))
                pw.add_variable(new, "lon_new", lon_new, ("xi_new",))
            # Identify variables that arn't coordinates, and their native grids,
            # convert non-rho variables to rho with `average_adjacents`. Create
            # sequence of rho-based variables in `roms_variables` object
            for key in nc.variables:
                var = nc.variables[key]
                try:
                    if key == "w":
                        if z == None:
                            z = depth_w
                        grid_type = "rho"
                    else:
                        if "_psi" in var.coordinates:
                            grid_type = "psi"
                        elif "_u" in var.coordinates:
                            grid_type = "u"
                        elif "_v" in var.coordinates:
                            grid_type = "v"
                        else:
                            grid_type = "rho"
                except AttributeError:
                    grid_type = None
                if grid_type != None and grid_type != "psi":
                    if key in set([ "sustr", "bustr", "DU_avg1", "DU_avg2", "u", "w", "ubar", "mask_u", "mask_v", "mask_psi", "mask_rho" ]):
                        pass
                    elif grid_type == "v":
                        var_dimensionality = len(var.shape)
                        if "sv" in key or "bv" in key or "DV" in key or key == "vbar" or key == "v":
                            paired_vector = {"svstr":"sustr", 
                                             "bvstr":"bustr", 
                                             "DV_avg1":"DU_avg1", 
                                             "DV_avg2":"DU_avg2",
                                             "vbar":"ubar",
                                             "v":"u"}
                            for i in xrange(var.shape[0]):
                                if var_dimensionality == 3:
                                    complex_uv = _uv_to_rho(nc.variables[paired_vector[key]][i,:,:], var[i,:,:], nc.variables["angle"][:], rho_x, rho_y)
                                    uz, vz = complex_uv.real, complex_uv.imag
                                elif var_dimensionality == 4:
                                    for j in xrange(var.shape[1]):
                                        complex_uv = _uv_to_rho(nc.variables["u"][i,j,:,:], var[i,j,:,:], nc.variables["angle"][:], rho_x, rho_y)
                                        if j == 0:
                                            uz, vz = complex_uv.real[np.newaxis,:], complex_uv.imag[np.newaxis,:]
                                        else:
                                            uz = np.vstack((uz, complex_uv.real[np.newaxis,:]))
                                            vz = np.vstack((vz, complex_uv.imag[np.newaxis,:]))
                                if i == 0:
                                    u, v = uz[np.newaxis,:], vz[np.newaxis,:]
                                else:
                                    u = np.vstack((u, uz[np.newaxis,:]))
                                    v = np.vstack((v, vz[np.newaxis,:]))
                            values = {paired_vector[key]:u, key:v}
                            for new_key in values:
                                values_interp[:, np.where(nc.variables["mask_rho"]==0)] = np.nan
                                if var_dimensionality == 3:
                                    interpolator = CfGeoInterpolator(values[new_key], lon_rho, lat_rho, t=time)
                                    values_interp = interpolator.interpgrid(lon_new, lat_new, t=t)
                                    coordtuple = ("time_new", "eta_new", "xi_new",)
                                    coordattr = "ocean_time lat_new lon_new"
                                elif var_dimensionality == 4:
                                    interpolator = CfGeoInterpolator(values[new_key], lon_rho, lat_rho, t=time, z=depth_rho)
                                    values_interp = interpolator.interpgrid(lon_new, lat_new, t=t, z=z)
                                    coordtuple = ("time_new", "s_new", "eta_new", "xi_new",)
                                    coordattr = "ocean_time s_new lat_new lon_new"
                                values_interp = np.ma.MaskedArray(values_interp, mask=values_interp==np.nan)
                                pw.add_variable(new, new_key, values_interp, coordtuple)
                                [pw.add_attribute(new, at, nc.variables[new_key].getncattr(at), var=new_key) for at in nc.variables[new_key].ncattrs()]
                                pw.add_attribute(new, "coordinates", coordattr, var=new_key)
                    else:
                        var_dimensionality = len(var.shape)
                        vartmp = var[:]
                        if var_dimensionality == 4:
                            vartmp[:,:, np.where(nc.variables["mask_rho"]==0)] = np.nan
                            interpolator = CfGeoInterpolator(vartmp, lon_rho, lat_rho, t=time, z=depth_rho)
                            values_interp = interpolator.interpgrid(lon_new, lat_new, t=t, z=z)
                            pw.add_variable(new, key, values_interp, ("time_new", "s_new", "eta_new", "xi_new",))
                            [pw.add_attribute(new, at, nc.variables[key].getncattr(at), var=key) for at in nc.variables[key].ncattrs()]
                            pw.add_attribute(new, "coordinates", "ocean_time s_new lat_new lon_new", var=key)
                        elif var_dimensionality == 3:
                            vartmp[:, np.where(nc.variables["mask_rho"]==0)] = np.nan
                            if var.shape[0] == time.shape[0]:
                                try:
                                    interpolator = CfGeoInterpolator(vartmp, lon_rho, lat_rho, t=time)
                                    values_interp = interpolator.interpgrid(lon_new, lat_new, t=t)
                                    values_interp = np.ma.MaskedArray(values_interp, mask=values_interp==np.nan)
                                    pw.add_variable(new, key, values_interp, ("time_new", "eta_new", "xi_new",))
                                    [pw.add_attribute(new, at, nc.variables[key].getncattr(at), var=key) for at in nc.variables[key].ncattrs()]
                                    pw.add_attribute(new, "coordinates", "ocean_time lat_new lon_new", var=key)
                                except:
                                    print key, vartmp.shape, lon_rho.shape, lat_rho.shape, time.shape
                            elif var.shape[0] == depth_rho.shape[0]:
                                interpolator = CfGeoInterpolator(vartmp, lon_rho, lat_rho, z=depth_rho)
                                values_interp = interpolator.interpgrid(lon_new, lat_new, z=z)
                                values_interp = np.ma.MaskedArray(values_interp, mask=values_interp==np.nan)
                                pw.add_variable(new, key, values_interp, ("depth_new", "eta_new", "xi_new",))
                                [pw.add_attribute(new, at, nc.variables[key].getncattr(at), var=key) for at in nc.variables[key].ncattrs()]
                                pw.add_attribute(new, "coordinates", "s_new lat_new lon_new", var=key)
                            else:
                                raise valueerror("unsure about what dimension this varaible varies with in addition to lat/lon.")
                        elif var_dimensionality == 2:
                            vartmp[np.where(nc.variables["mask_rho"]==0)] = np.nan
                            interpolator = CfGeoInterpolator(vartmp, lon_rho, lat_rho)
                            values_interp = interpolator.interpgrid(lon_new, lat_new)
                            values_interp = np.ma.MaskedArray(values_interp, mask=values_interp==np.nan)
                            pw.add_variable(new, key, values_interp, ("eta_new", "xi_new",))
                            [pw.add_attribute(new, at, nc.variables[key].getncattr(at), var=key) for at in nc.variables[key].ncattrs()]
                            pw.add_attribute(new, "coordinates", "lat_new lon_new", var=key)
                        else:
                            # todo if 1-d check for which dimension it matches and interp based on that...
                            #      if 5+ d, only interpolate to the 4d dimensions that we can specify i guess...
                            raise valueerror("sort of confused about the dimensionality of the variable i am attempting to regrid...")
                elif grid_type == "psi":
                    pass
                else:
                    if len(var.dimensions) == 0:
                        pw.add_scalar(new, key, var[:])
                        [pw.add_attribute(new, at, nc.variables[key].getncattr(at), var=key) for at in nc.variables[key].ncattrs()]

            # Add time attributes
            for key in nc.variables:
                var = nc.variables[key]
                if "time" in key:
                    [pw.add_attribute(new, at, nc.variables[key].getncattr(at), var="ocean_time") for at in nc.variables[key].ncattrs()]

            # Add global attributes to the file
            [pw.add_attribute(new, at, nc.getncattr(at)) for at in nc.ncattrs()]
            try:
                new.history = new.history + ", regridded by Python tool 'paegan' at " + str(datetime.datetime.now())
            except:
                new.history = "regridded by Python tool 'paegan' at " + str(datetime.datetime.now())
        new.sync()

# Threaded instance for speed testing on enormous grids.
class AverageAdjacents(threading.Thread):
    def __init__(self, data, by_column=False):
        threading.Thread.__init__(self)
        self.by_column = by_column
        self.data = data
    def run(self):
        if self.by_column:
            # Easier to transpose, sum by row, and transpose back.
            #sumd = 0.5 * (a[1:m,:] + a[0:m-1,:])
            t3 = AverageAdjacents(self.data.T)
            t3.start(); t3.join()
            self.data = t3.data.T
            return

        try:
            [m,n] = self.data.shape
        except ValueError:
            [m,n] = [1,self.data.size]

        if m == 1:
            sumd = 0.5 * (self.data[0:n-1] + self.data[1:n]) # Single row
        else:
            sumd = 0.5 * (self.data[:,0:n-1] + self.data[:,1:n]) # By row

        self.data = sumd
        return




from shapely.geometry import MultiPoint

from paegan.cdm.dsg.collections.base.profile_collection import ProfileCollection
from paegan.utils.asalist import AsaList

class StationProfile(ProfileCollection):
    """
    A collection of Profiles at a single location.

    This mimics the Station class API, they should likely be merged somehow.
    """
    def __init__(self, **kwargs):
        super(StationProfile, self).__init__(**kwargs)

        self._type       = "timeSeriesProfile"
        self._properties = {}
        self._location   = None
        self.uid         = None
        self.name        = None
        self.description = None

    @property
    def location(self):
        return self._location

    @location.setter
    def location(self, value):
        self._location = value

        # Sets the location of every Profile in this station if it is not already set
        for profile in self._elements:
            if profile.location is None:
                profile.location == value

    def get_properties(self):
        return self._properties

    def get_property(self, prop):
        return self._properties.get(prop, None)

    def set_property(self, prop, value):
        self._properties[prop] = value

    def get_unique_members(self):
        all_members = [pt.members for prof in self._elements
                                  for pt in prof._elements]

        keys = ["name", "description", "standard", "units"]
        mwhat = []
        for mg in all_members:
            for m in mg:
                mwhat.append( { key: m[key] for key in keys if key in m } )

        # Now unique them on name
        mwhat = { x['name']:x for x in mwhat }.values()

        return mwhat

    def calculate_bounds(self):
        """
            Calculate the time_range, depth_range, bbox, and size of this collection.
            Will scan all data.
            Ensures that .size, .bbox and .time_range return non-null.

            If the collection already knows its bbox; time_range; and/or size,
            they are recomputed.
        """
        # tell all contained profiles to calculate their bounds
        map(lambda x: x.calculate_bounds(), self._elements)

        # @TODO size is just number of timesteps?
        self.size = len(self._elements)

        # bbox is just this point
        self.bbox = MultiPoint([self.location, self.location]).envelope

        time_set = set()
        map(time_set.add, AsaList.flatten([p.time for p in self._elements]))
        self.time_range = sorted(list(time_set))

        depth_set = set()
        map(depth_set.add, AsaList.flatten([p.depth_range for p in self._elements]))
        self.depth_range = sorted(list(depth_set))






import numpy as np
from scipy.interpolate import griddata

def create_grid(lonmin, lonmax, latmin, latmax, **kwargs):
    dx, dy = kwargs.get("dx", None), kwargs.get("dy", None)
    nx, ny = kwargs.get("nx", None), kwargs.get("ny", None)
    if nx==None or ny==None:
        if dx==None or dy==None:
            raise KeyError("Please kwargs dx and dy to define the interval between grid points, or nx and ny to define the number of grid points (include endpoints) between the bbox limits")
        else:
            nx, ny = (lonmax-lonmin)/dx, (latmax-latmin)/dy
    lat_grid = np.linspace(latmin, latmax, ny, endpoint=True)
    lon_grid = np.linspace(lonmin, lonmax, nx, endpoint=True)
    return lon_grid, lat_grid

class Interpolator(object):
    def __init__(self, *dimensions, **kwargs):
        dimensions = [dim.flatten() for dim in dimensions]
        points = [dim.flatten() for dim in np.meshgrid(*dimensions)]
        points = np.asarray(points).T
        self.points = points
        self.data = data.flatten()
        self.method = kwargs.get('method', 'nearest')
        self.numdim = len(dimensions)

    def interpgrid(self, *dimensions):
        if len(dimensions) != self.numdim:
            raise ValueError("Please interpolate data to the same number of dimensions as source data.")
        else:
            flatdims = [dim.flatten() for dim in dimensions]
            newpoints = [dim.flatten() for dim in np.meshgrid(*flatdims)]
            newpoints = np.asarray(newpoints).T
            f = griddata(self.points, self.data, newpoints, method=self.method)
            unique = []
            for dim in dimensions:
                unique.append(dim.shape)
            unique = np.asarray(unique).unique()
            return np.squeeze( f.reshape( *unique ) )

